#include "memos-app.h"
#include "x86.h"
#include "x86struct.h"
#include "x86mem.h"

/*****************************************************************************
 * memos-1
 *
 *   This tiny application gradually allocates its heap space, one page
 *   a time, until it runs out of space or the machine runs out of physical
 *   memory.
 *
 *   The other memos-* processes simply #include this file after defining
 *   ALLOC_PRIORITY appropriately.
 *
 *****************************************************************************/

// How fast should we allocate heap space?  Higher numbers are faster.
#ifndef ALLOC_PRIORITY
#define ALLOC_PRIORITY	1
#endif

extern uint8_t end;

void
start(void)
{
	uint8_t *heap_top, *stack_bottom;

	// The heap starts on the page right after the 'end' symbol,
	// whose address is the first address not allocated to process code
	// or data
	heap_top = ROUNDUP(&end, PAGESIZE);

	// The bottom of the stack is the first address on the current
	// stack page (this process never needs more than one stack page)
	stack_bottom = ROUNDDOWN((uint8_t *) read_esp(), PAGESIZE);

	// Allocate heap pages until (1) hit the stack (out of address space)
	// or (2) allocation fails (out of physical memory)
	while (1) {
		if ((sys_random() % (4 + ALLOC_SPEED)) < ALLOC_PRIORITY) {
			if (heap_top == stack_bottom
			    || sys_page_alloc(heap_top) < 0)
				break;
			heap_top += PAGESIZE;
		}
		sys_yield();
	}

	// After running out of memory, do nothing forever
	while (1)
		sys_yield();
}
