#ifndef WEENSYOS_MEMOS_KERN_H
#define WEENSYOS_MEMOS_KERN_H
#include "memos.h"
#include "x86struct.h"
#include "lib.h"

// PROCESS DESCRIPTOR TYPE
typedef struct process {
	registers_t p_registers;	// Current process state: registers,
					// stack location, EIP, etc.
					// 'registers_t' defined in x86struct.h
	int p_pid;			// Process ID
	int p_runnable;			// Is the process runnable?
	pagedirectory_t p_pgdir;	// Process's page directory
} process_t;

#define NPROCESSES		4	// Number of processes
extern process_t processes[NPROCESSES];	// Process descriptor array
extern process_t *current_proc;


// MEMORY CONSTANTS
#define KERNEL_START		0x00040000	// Start of kernel code
#define KERNEL_STACK_TOP	0x00080000	// Top of kernel stack

#define PROCESS_START		0x00100000	// Start of PID 0 process code
#define PROCESS_SIZE		0x00040000	// Process size
						// (physical memory only)

// Functions defined in memos-kern.c
void trap(registers_t *reg);
void schedule(void) __attribute__((noreturn));

// Initialization functions (memos-x86.c)
void segments_init(void);
void interrupt_controller_init(void);
void process_init(process_t *proc);
void process_load(process_t *proc);

// Page functions defined in memos-pages.c
pagedirectory_t pgdir_new(void);
void paged_virtual_memory_init(pagedirectory_t pgdir);
int page_alloc(process_t *proc, uint32_t va);

// Memory label functions (memos-pages.c)
void memory_labels_init(void);
void show_physical_memory(void);
void show_virtual_memory(process_t *virtual_address_space);
void show_virtual_memory_flipper(void);

// Console functions (console.c)
void console_clear(void);
void console_printf(int row, int col, const char *format, ...);
void console_vprintf(int row, int col, const char *format, va_list val);
void panic(const char *message, ...) __attribute__((noreturn));

#endif
